--------------------------------------------------------------------------------
-- -- Copyright (c) 2009 Xilinx, Inc.
-- -- This design is confidential and proprietary of Xilinx, All Rights
-- Reserved.
--------------------------------------------------------------------------------
-- -   ____  ____
-- -  /   /\/   /
-- - /___/  \  /   Vendor: Xilinx
-- - \   \   \/    Version: 1.0
-- -  \   \        Filename: BUS_ALIGN_MACHINE.vhd
-- -  /   /        
-- - /___/   /\    Date Created: 07/14/2009 
-- - \   \  /  \   
-- -  \___\/\___\
-- - 
-- -Revision History:
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- Description of module:
--
-- The BUS_ALIGN_MACHINE module samples the clock input as if it were a data 
-- channel, and uses the information about the deserialized clock to determine 
-- the optimal clock/data relationship for that channel.  By dynamically 
-- changing the delay of the clock channel (with respect to the data bus), the 
-- machine places the sampling point at the center of the data eye.
--  
--------------------------------------------------------------------------------

----
--
-- Library declarations
--
-- Standard IEEE libraries
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library unisim;
use unisim.vcomponents.all;
--

ENTITY BUS_ALIGN_MACHINE IS
   PORT (
      RXCLKDIV                : IN std_logic;   -- PARALLEL RX DOMAIN CLOCK
      RST                     : IN std_logic;   
                         -- RESET SYNCHRONOUS TO Rx DOMAIN
      SAMPLED_CLOCK           : IN std_logic_vector(3 DOWNTO 0);   
                         -- 4 CLOCK CHANNEL SAMPLES
      INC                     : OUT std_logic;   
                         -- COMMAND TO INC/DEC DELAY OF CLOCK CHANNEL
      ICE                     : OUT std_logic;   
                         -- COMMAND TO ENABLE DELAY ADJUSTMENT OF CLOCK CHANNEL
      DATA_ALIGNED            : OUT std_logic);   
                         -- FLAG INDICATING THAT THE BUS IS ALIGNED
END BUS_ALIGN_MACHINE;

ARCHITECTURE BUS_ALIGN_MACHINE OF BUS_ALIGN_MACHINE IS

------------------------------------------------------------------------------------
--
-- Component Declaration

component count_to_128 
   PORT (
      clk                     : IN std_logic;   
      rst                     : IN std_logic;   
      count                   : IN std_logic;   
      ud                      : IN std_logic;   
      counter_value           : OUT std_logic_vector(6 DOWNTO 0));   
end component;


   SIGNAL COUNT                    :  std_logic;   
   SIGNAL UD                       :  std_logic;   
   SIGNAL COUNT_SAMPLE             :  std_logic;   
   SIGNAL UD_SAMPLE                :  std_logic;   
   SIGNAL CURRENT_STATE            :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL NEXT_STATE               :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL COUNT_VALUE              :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL SAMPLED_CLOCK_PREV       :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL COUNT_VALUE_SAMPLE       :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL INC_TEMP                      :  std_logic;   
   SIGNAL ICE_TEMP                      :  std_logic;   
   SIGNAL DATA_ALIGNEDx            :  std_logic;   

BEGIN


INC <= INC_TEMP;
ICE <= ICE_TEMP;

   machine_counter_total : count_to_128 
      PORT MAP (
         clk => RXCLKDIV,
         rst => RST,
         count => COUNT_SAMPLE,
         ud => UD_SAMPLE,
         counter_value => COUNT_VALUE_SAMPLE);   
   

   machine_counter : count_to_128 
      PORT MAP (
         clk => RXCLKDIV,
         rst => RST,
         count => COUNT,
         ud => UD,
         counter_value => COUNT_VALUE);   
   
   count_reg : FDR 
      PORT MAP (
         Q => DATA_ALIGNED,
         C => RXCLKDIV,
         D => DATA_ALIGNEDx,
         R => RST);   
   
   bit0 : FDRE 
      PORT MAP (
         Q => SAMPLED_CLOCK_PREV(0),
         C => RXCLKDIV,
         CE => '1',
         D => SAMPLED_CLOCK(0),
         R => RST);   
   
   bit1 : FDRE 
      PORT MAP (
         Q => SAMPLED_CLOCK_PREV(1),
         C => RXCLKDIV,
         CE => '1',
         D => SAMPLED_CLOCK(1),
         R => RST);   
   
   bit2 : FDRE 
      PORT MAP (
         Q => SAMPLED_CLOCK_PREV(2),
         C => RXCLKDIV,
         CE => '1',
         D => SAMPLED_CLOCK(2),
         R => RST);   
   
   bit3 : FDRE 
      PORT MAP (
         Q => SAMPLED_CLOCK_PREV(3),
         C => RXCLKDIV,
         CE => '1',
         D => SAMPLED_CLOCK(3),
         R => RST);   
   
   DATA_ALIGNEDx <= (((NOT CURRENT_STATE(3)) AND CURRENT_STATE(2)) AND 
   CURRENT_STATE(1)) AND (CURRENT_STATE(0)) ;

   PROCESS (RXCLKDIV, RST)
   BEGIN
      IF (RST = '1') THEN  --Initial Values     
         CURRENT_STATE <= "0000";    
      ELSIF (RXCLKDIV'EVENT AND RXCLKDIV = '1') THEN --Transition Values         
         CURRENT_STATE <= NEXT_STATE;    
      END IF;
      --CURRENT_STATE <= CURRENT_STATE;
   END PROCESS;

   --NEXT_STATE LOGIC
   
   PROCESS (CURRENT_STATE, SAMPLED_CLOCK, COUNT_VALUE, 
   SAMPLED_CLOCK_PREV, COUNT_VALUE_SAMPLE)
   BEGIN
      CASE CURRENT_STATE IS
         WHEN "0000" =>
                  NEXT_STATE <= "0001";    -- RST STATE	
         WHEN "0001" =>          --INITIAL STATE, SAMPLE TRAINING BIT
                  IF (SAMPLED_CLOCK_PREV /= SAMPLED_CLOCK) THEN
                     NEXT_STATE <= "1111";    
                  ELSE
                     NEXT_STATE <= "1000";    
                  END IF;
         WHEN "1000" =>    --CHECK SAMPLE TO SEE IF IT IS ON A TRANSITION
                  IF (SAMPLED_CLOCK_PREV /= SAMPLED_CLOCK) THEN
                     NEXT_STATE <= "1111";    
                  ELSE
                     IF ((COUNT_VALUE > "0001111") AND (SAMPLED_CLOCK = 
                     "1111")) THEN
                        NEXT_STATE <= "1011";    
                     ELSE
                        IF ((COUNT_VALUE > "0001111") AND (SAMPLED_CLOCK 
                        = "0000")) THEN
                           NEXT_STATE <= "0010";    
                        ELSE
                           NEXT_STATE <= "1000";    
                        END IF;
                     END IF;
                  END IF;
         WHEN "1111" =>
                  --IF SAMPLED POINT IS TRANSITION (INSTABLE), INCREMENT DELAY 3 TIMES
                  
                  IF (COUNT_VALUE_SAMPLE > "0000001") THEN
                     NEXT_STATE <= "1101";    
                  ELSE
                     NEXT_STATE <= "1111";    
                  END IF;
         WHEN "1101" =>
                  --WAIT 8 CYCLES
                  
                  IF (COUNT_VALUE > "0000111") THEN
                     NEXT_STATE <= "1100";    
                  ELSE
                     NEXT_STATE <= "1101";    
                  END IF;
         WHEN "1100" =>
                  --IDLE (NEEDED FOR COUNTER RESET)
                  
                  NEXT_STATE <= "1000";    
         WHEN "0010" =>
                  --INC ONCE
                  
                  NEXT_STATE <= "1110";    
         WHEN "1110" =>
                  --WAIT 8 CYCLES, LOOK FOR RISING EDGE
                  
                  IF (COUNT_VALUE_SAMPLE < "0000111") THEN
                     NEXT_STATE <= "1110";    
                  ELSE
                     IF (SAMPLED_CLOCK /= "0000") THEN
                        NEXT_STATE <= "0111";    
                     ELSE
                        NEXT_STATE <= "0010";    
                     END IF;
                  END IF;
         WHEN "1011" =>
                  --INC ONCE
                  
                  NEXT_STATE <= "0100";    
         WHEN "0100" =>
                  --WAIT 8 CYCLES, LOOK FOR FALLING EDGE
                  
                  IF (COUNT_VALUE_SAMPLE < "0000111") THEN
                     NEXT_STATE <= "0100";    
                  ELSE
                     IF (SAMPLED_CLOCK = "0000") THEN
                        NEXT_STATE <= "1001";    
                     ELSE
                        NEXT_STATE <= "1011";    
                     END IF;
                  END IF;
         WHEN "1001" =>
                  --INC ONCE
                  
                  NEXT_STATE <= "0011";    
         WHEN "0011" =>
                  --WAIT 8 CYCLES, LOOK FOR RISING EDGE
                  
                  IF (COUNT_VALUE_SAMPLE < "0000111") THEN
                     NEXT_STATE <= "0011";    
                  ELSE
                     IF (SAMPLED_CLOCK /= "0000") THEN
                        NEXT_STATE <= "0111";    
                     ELSE
                        NEXT_STATE <= "1001";    
                     END IF;
                  END IF;
         WHEN "0111" =>
                  NEXT_STATE <= "0111";    -- TRAINING COMPLETE FOR THIS CHANNEL			
         WHEN OTHERS  =>
                  NEXT_STATE <= "0000";    
         
      END CASE;
   END PROCESS;

   --OUTPUT LOGIC
   
   PROCESS (CURRENT_STATE)

   BEGIN
      CASE CURRENT_STATE IS
         WHEN "0000" =>
                  --RST STATE
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "0001" =>
                  --INITIAL STATE, SAMPLE TRAINING BIT
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "1000" =>
                  --CHECK SAMPLE TO SEE IF IT IS ON A TRANSITION
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '1';    
                  UD  <= '1';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "1111" =>
                  --IF SAMPLED POINT IS TRANSITION, INCREMENT DELAY 2 TIMES
                  
                  INC_TEMP  <= '1';    
                  ICE_TEMP  <= '1';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '1';    
                  UD_SAMPLE  <= '1';    
         WHEN "1101" =>
                  --WAIT 8 CYCLES
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '1';    
                  UD  <= '1';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "1100" =>
                  --IDLE (NEEDED FOR COUNTER RESET)
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "0010" =>
                  --INC ONCE
                  
                  INC_TEMP  <= '1';    
                  ICE_TEMP  <= '1';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "1110" =>
                  --WAIT 8 CYCLES, LOOK FOR 1 (TO LOCATE FALLING EDGE)
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '1';    
                  UD_SAMPLE  <= '1';    
         WHEN "1011" =>
                  --INC ONCE
                  
                  INC_TEMP  <= '1';    
                  ICE_TEMP  <= '1';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "0100" =>
                  --WAIT 8 CYCLES, LOOK FOR 0
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '1';    
                  UD_SAMPLE  <= '1';    
         WHEN "1001" =>
                  --INC ONCE
                  
                  INC_TEMP  <= '1';    
                  ICE_TEMP  <= '1';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN "0011" =>
                  --WAIT 8 CYCLES, LOOK FOR 1 (TO LOCATE FALLING EDGE)
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '1';    
                  UD_SAMPLE  <= '1';    
         WHEN "0111" =>
                  --TRAINING COMPLETE ON THIS CHANNEL
                  
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         WHEN OTHERS  =>
                  INC_TEMP  <= '0';    
                  ICE_TEMP  <= '0';    
                  COUNT  <= '0';    
                  UD  <= '0';    
                  COUNT_SAMPLE  <= '0';    
                  UD_SAMPLE  <= '0';    
         
      END CASE;
   END PROCESS;

END BUS_ALIGN_MACHINE;
