--------------------------------------------------------------------------------
-- -- Copyright (c) 2009 Xilinx, Inc.
-- -- This design is confidential and proprietary of Xilinx, All Rights
-- Reserved.
--------------------------------------------------------------------------------
-- -   ____  ____
-- -  /   /\/   /
-- - /___/  \  /   Vendor: Xilinx
-- - \   \   \/    Version: 1.0
-- -  \   \        Filename: SDR_4TO1_16CHAN_RX.vhd
-- -  /   /        
-- - /___/   /\    Date Created: 07/14/2009 
-- - \   \  /  \   
-- -  \___\/\___\
-- - 
-- -Revision History:
--------------------------------------------------------------------------------
-- 
-- Description of module:
--
-- The SDR_4TO1_16CHAN_RX module contains all logic in the LVDS Receiver,
-- including 16 channels of LVDS data, one channel of LVDS clock, a 
-- clock/data alignment algorithm. 
--  
--------------------------------------------------------------------------------

----
--
-- Library declarations
--
-- Standard IEEE libraries
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library unisim;
use unisim.vcomponents.all;
--


ENTITY SDR_4TO1_16CHAN_RX IS
   PORT (
      DATA_RX_P               : IN std_logic_vector(15 DOWNTO 0);   
                                  -- SERIAL SIDE RX DATA (P)
      DATA_RX_N               : IN std_logic_vector(15 DOWNTO 0);   
                                  -- SERIAL SIDE RX DATA (N)
      CLOCK_RX_P              : IN std_logic;   -- FORWARDED CLOCK FROM TX (P)
      CLOCK_RX_N              : IN std_logic;   -- FORWARDED CLOCK FROM TX (N)
      INC_PAD                 : IN std_logic;   
                                  -- MANUAL INCREMENT TO DATA DELAY
      DEC_PAD                 : IN std_logic;   
                                  -- MANUAL DECREMENT TO DATA DELAY
      DATA_FROM_ISERDES       : OUT std_logic_vector(63 DOWNTO 0);   
                                  -- PARALLEL SIDE RX DATA
      RESET                   : IN std_logic;   -- RX DOMAIN RESET
      IDLY_RESET              : IN std_logic;   -- IDELAY TAP RESET
      IDELAYCTRL_RESET        : IN std_logic;   -- IDELAYCTRL CIRCUIT RESET
      CLK200                  : IN std_logic;   
                                  -- 200 MHZ REFERENCE CLOCK TO IDELAYCTRL
      TAP_00                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_01                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_02                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_03                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_04                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_05                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_06                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_07                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_08                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_09                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_10                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_11                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_12                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_13                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_14                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_15                  : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT (0-31)
      TAP_CLK                 : OUT std_logic_vector(4 DOWNTO 0);   
                                  -- IDELAY TAP COUNT ON CLK CHANNEL (0-31)
      TRAINING_DONE           : OUT std_logic;   
                                  -- ALIGNMENT OF ALL CHANNELS COMPLETE
      RXCLK                   : OUT std_logic;   
                                  -- FORWARDED CLOCK FROM TX (BUFIO OUTPUT)
      RXCLKDIV                : OUT std_logic;   
                                  -- PARALLEL SIDE RX CLOCK (DIVIDED FROM RXCLK)
      IDELAY_READY            : OUT std_logic);   
                                  -- FLAG INDICATING THAT IDELAYCTRL IS LOCKED
END SDR_4TO1_16CHAN_RX;

ARCHITECTURE SDR_4TO1_16CHAN_RX OF SDR_4TO1_16CHAN_RX IS

   COMPONENT BUS_ALIGN_MACHINE
      PORT (
         RXCLKDIV                : IN  std_logic;
         RST                     : IN  std_logic;
         SAMPLED_CLOCK           : IN  std_logic_vector(3 DOWNTO 0);
         INC                     : OUT std_logic;
         ICE                     : OUT std_logic;
         DATA_ALIGNED            : OUT std_logic);
   END COMPONENT;


   SIGNAL CLOCK_RX_BUF             :  std_logic;   
   SIGNAL DATA_RX_BUF              :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL CLOCK_RX_IODELAY_OUT     :  std_logic;  
   SIGNAL CLOCK_RX_IODELAY_TEMP    :  std_logic; 
   SIGNAL CLOCK_RX_ISERDES_OUT     :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL INC_CAPTURE              :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL DEC_CAPTURE              :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL INC_PULSE                :  std_logic;   
   SIGNAL DEC_PULSE                :  std_logic;   
   SIGNAL RESET_SM                 :  std_logic_vector(15 DOWNTO 0);   
                               --  Reset pipe used for Data Iserdes reset
   SIGNAL I                        :  integer;   
   SIGNAL K                        :  integer;   

   SIGNAL RXCLK_TEMP               :  std_logic;
   SIGNAL RXCLK_TMP_INV            :  std_logic;   
   SIGNAL RXCLKDIV_TEMP            :  std_logic;   
   SIGNAL ICE_DELAY                :  std_logic;   
   SIGNAL ICE_TO_IODELAY_CLK       :  std_logic;   
   SIGNAL INC_DELAY                :  std_logic;   
   SIGNAL INC_TO_IODELAY_CLK       :  std_logic;   
   SIGNAL IODELAY_CE               :  std_logic;
   SIGNAL IODELAY_INC              :  std_logic;
   SIGNAL IODELAY_RESET            :  std_logic;
   SIGNAL BUS_ALIGN_RST            :  std_logic;

   ATTRIBUTE KEEP : STRING;
   ATTRIBUTE KEEP OF CLOCK_RX_IODELAY_OUT: SIGNAL IS "TRUE";
BEGIN


   RXCLK <= RXCLK_TEMP;
   RXCLKDIV <= RXCLKDIV_TEMP;

   INC_DELAY <= INC_PULSE ;
   ICE_DELAY <= INC_PULSE OR DEC_PULSE ;
   TAP_00  <= "00000" ;
   TAP_01  <= "00000" ;
   TAP_02  <= "00000" ;
   TAP_03  <= "00000" ;
   TAP_04  <= "00000" ;
   TAP_05  <= "00000" ;
   TAP_06  <= "00000" ;
   TAP_07  <= "00000" ;
   TAP_08  <= "00000" ;
   TAP_09  <= "00000" ;
   TAP_10  <= "00000" ;
   TAP_11  <= "00000" ;
   TAP_12  <= "00000" ;
   TAP_13  <= "00000" ;
   TAP_14  <= "00000" ;
   TAP_15  <= "00000" ;
   
   --IDELAYCTRL MODULE

   RX_IDELAYCTRL : IDELAYCTRL 
      PORT MAP (
         RDY => IDELAY_READY,
         REFCLK => CLK200,
         RST => IDELAYCTRL_RESET);   
   
      --SOURCE SYNCHRONOUS CLOCK INPUT

   SOURCE_SYNC_CLOCK_IN : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => CLOCK_RX_BUF,
         I => CLOCK_RX_P,
         IB => CLOCK_RX_N);   

 
IODELAY_CE <= ICE_DELAY OR ICE_TO_IODELAY_CLK ;
IODELAY_INC <= INC_DELAY OR INC_TO_IODELAY_CLK;
IODELAY_RESET <= IDLY_RESET OR RESET ;
RXCLK_TMP_INV <= not (RXCLK_TEMP);

   --CLOCK BUFFER FOR SERIAL SIDE CLOCK
   RX_CLK_BUFIO : BUFIO 
      PORT MAP (
         O => RXCLK_TEMP,
         I => CLOCK_RX_IODELAY_OUT);   
   
   
   --CLOCK BUFFER/DIVIDER FOR PARALLEL SIDE CLOCK
   RX_CLK_BUFR : BUFR 
      GENERIC MAP( 
         BUFR_DIVIDE => "4" )
      PORT MAP (
         O => RXCLKDIV_TEMP,
         CE => '1',
         CLR => '0',
         I => CLOCK_RX_IODELAY_OUT);   


   --  CLOCK DESERIALIZATION TO MIMIC DATA PATH
   ISERDES_inst : ISERDESE1 
   GENERIC MAP(
      DATA_RATE => "SDR",             --  Specify data rate of "DDR" or "SDR
      DATA_WIDTH => 4,                --  Specify data width - For DDR 4,6,8, or 10
                                      --  For SDR 2,3,4,5,6,7, or 8 
      INIT_Q1 => '0',                 -- INIT for Q1 reg - 1'b1 or 1'b0
      INIT_Q2 => '0',                 -- INIT for Q2 reg - 1'b1 or 1'b0
      INIT_Q3 => '0',                 -- INIT for Q3 reg - 1'b1 or 1'b0 
      INIT_Q4 => '0',                 -- INIT for Q4 reg - 1'b1 or 1'b0
      INTERFACE_TYPE => "NETWORKING", --  Use model - "MEMORY" or "NETWORKING
      IOBDELAY => "IBUF",             -- Specify outputs where delay chain will be  
                                      -- applied "NONE", "IBUF", "IFD", or "BOTH
      OFB_USED => FALSE,              -- Enables the use of D input 
      NUM_CE => 1,                    -- Define number or clock enables to an integer of 1 or 2
      SERDES_MODE => "MASTER"         -- Set SERDES mode to "MASTER" or "SLAVE
      )
      PORT MAP (
         O  => CLOCK_RX_IODELAY_OUT,
         Q1 => CLOCK_RX_ISERDES_OUT(003),
         Q2 => CLOCK_RX_ISERDES_OUT(002),
         Q3 => CLOCK_RX_ISERDES_OUT(001),
         Q4 => CLOCK_RX_ISERDES_OUT(000),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => CLOCK_RX_BUF,
         DDLY => CLOCK_RX_IODELAY_TEMP,
         DYNCLKDIVSEL => '0',   
         DYNCLKSEL => '0',   
         OFB => '0',    
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   
  
   --IDELAY IN CLOCK PATH
   IODELAY_CLOCK_RX: IODELAYE1 
     GENERIC MAP(
        IDELAY_TYPE => "VARIABLE", 
        IDELAY_VALUE => 0, 
        ODELAY_VALUE => 0,
        REFCLK_FREQUENCY => 200.00,  
        DELAY_SRC => "I",
        HIGH_PERFORMANCE_MODE => TRUE,
        SIGNAL_PATTERN => "CLOCK") 
      PORT MAP (
        DATAOUT => CLOCK_RX_IODELAY_TEMP,
        IDATAIN => CLOCK_RX_BUF,
        ODATAIN => '0',
        DATAIN => '0',
        T => '0',
        CINVCTRL => '0',    
        CLKIN => '0',
        CNTVALUEIN => "00000",
        CNTVALUEOUT => TAP_CLK,               
        CE => IODELAY_CE,
        INC => IODELAY_INC,
        C => RXCLKDIV_TEMP,
        RST => IODELAY_RESET);   
   

   BUS_ALIGN_RST <= RESET OR RESET_SM(15);
  
   BUS_ALIGN_MACHINE_0 : BUS_ALIGN_MACHINE 
      PORT MAP (
         RXCLKDIV => RXCLKDIV_TEMP,
         RST => BUS_ALIGN_RST,
         SAMPLED_CLOCK => CLOCK_RX_ISERDES_OUT,
         INC => INC_TO_IODELAY_CLK,
         ICE => ICE_TO_IODELAY_CLK,
         DATA_ALIGNED => TRAINING_DONE);   
   

   --SHORTEN EACH EXTERNAL INC AND DEC PULSE TO ONE RXCLKDIV CYCLE
   
   PROCESS (RXCLKDIV_TEMP)
   BEGIN
      IF (RXCLKDIV_TEMP'EVENT AND RXCLKDIV_TEMP = '1') THEN
         INC_CAPTURE(0) <= INC_PAD;    -- ASYNCHRONOUS ENTRY POINT
         DEC_CAPTURE(0) <= DEC_PAD;    
         FOR I IN 0 TO 3 - 1 LOOP
            INC_CAPTURE(I + 1) <= INC_CAPTURE(I);    -- METASTABLE FLIP-FLOPS
            DEC_CAPTURE(I + 1) <= DEC_CAPTURE(I);    
         END LOOP;
         INC_PULSE <= INC_CAPTURE(2) AND NOT INC_CAPTURE(3);    -- STABLE, SINGLE PULSE
         DEC_PULSE <= DEC_CAPTURE(2) AND NOT DEC_CAPTURE(3);    
      END IF;
   END PROCESS;


   PROCESS (RXCLKDIV_TEMP)
   BEGIN
      IF (RXCLKDIV_TEMP'EVENT AND RXCLKDIV_TEMP = '1') THEN
         RESET_SM(0) <= RESET;    
         FOR K IN 0 TO 15 - 1 LOOP
            RESET_SM(K + 1) <= RESET_SM(K);    
         END LOOP;
      END IF;
   END PROCESS;
   
   --DATA INPUT BUFFERS

   RX_DATA_IN_00 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(00),
         I => DATA_RX_P(00),
         IB => DATA_RX_N(00));   
   

   RX_DATA_IN_01 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(01),
         I => DATA_RX_P(01),
         IB => DATA_RX_N(01));   
   

   RX_DATA_IN_02 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(02),
         I => DATA_RX_P(02),
         IB => DATA_RX_N(02));   
   
   
   RX_DATA_IN_03 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(03),
         I => DATA_RX_P(03),
         IB => DATA_RX_N(03));   
   

   RX_DATA_IN_04 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(04),
         I => DATA_RX_P(04),
         IB => DATA_RX_N(04));   
   

   RX_DATA_IN_05 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(05),
         I => DATA_RX_P(05),
         IB => DATA_RX_N(05));   
   
   
   RX_DATA_IN_06 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(06),
         I => DATA_RX_P(06),
         IB => DATA_RX_N(06));   
   

   RX_DATA_IN_07 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(07),
         I => DATA_RX_P(07),
         IB => DATA_RX_N(07));   
   

   RX_DATA_IN_08 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(08),
         I => DATA_RX_P(08),
         IB => DATA_RX_N(08));   
   

   RX_DATA_IN_09 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(09),
         I => DATA_RX_P(09),
         IB => DATA_RX_N(09));   
   

   RX_DATA_IN_10 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(10),
         I => DATA_RX_P(10),
         IB => DATA_RX_N(10));   
   

   RX_DATA_IN_11 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(11),
         I => DATA_RX_P(11),
         IB => DATA_RX_N(11));   
   

   RX_DATA_IN_12 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(12),
         I => DATA_RX_P(12),
         IB => DATA_RX_N(12));   
   

   RX_DATA_IN_13 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(13),
         I => DATA_RX_P(13),
         IB => DATA_RX_N(13));   
   

   RX_DATA_IN_14 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(14),
         I => DATA_RX_P(14),
         IB => DATA_RX_N(14));   
   

   RX_DATA_IN_15 : IBUFDS 
      GENERIC MAP(
         DIFF_TERM => TRUE, IOSTANDARD =>"LVDS_25") 
      PORT MAP (
         O => DATA_RX_BUF(15),
         I => DATA_RX_P(15),
         IB => DATA_RX_N(15));   
   
   
   --ISERDES IN DATA PATH                                                                                                    
   ISERDES_RX_DATA_00 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")
      PORT MAP (
         Q1 => DATA_FROM_ISERDES(048),
         Q2 => DATA_FROM_ISERDES(032),
         Q3 => DATA_FROM_ISERDES(016),
         Q4 => DATA_FROM_ISERDES(000),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(00),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   
   

   ISERDES_RX_DATA_01 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(049),
         Q2 => DATA_FROM_ISERDES(033),
         Q3 => DATA_FROM_ISERDES(017),
         Q4 => DATA_FROM_ISERDES(001),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(01),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_02 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(050),
         Q2 => DATA_FROM_ISERDES(034),
         Q3 => DATA_FROM_ISERDES(018),
         Q4 => DATA_FROM_ISERDES(002),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(02),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   
   
   ISERDES_RX_DATA_03 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(051),
         Q2 => DATA_FROM_ISERDES(035),
         Q3 => DATA_FROM_ISERDES(019),
         Q4 => DATA_FROM_ISERDES(003),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(03),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_04 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(052),
         Q2 => DATA_FROM_ISERDES(036),
         Q3 => DATA_FROM_ISERDES(020),
         Q4 => DATA_FROM_ISERDES(004),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(04),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_05 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(053),
         Q2 => DATA_FROM_ISERDES(037),
         Q3 => DATA_FROM_ISERDES(021),
         Q4 => DATA_FROM_ISERDES(005),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(05),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_06 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(054),
         Q2 => DATA_FROM_ISERDES(038),
         Q3 => DATA_FROM_ISERDES(022),
         Q4 => DATA_FROM_ISERDES(006),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(06),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_07 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(055),
         Q2 => DATA_FROM_ISERDES(039),
         Q3 => DATA_FROM_ISERDES(023),
         Q4 => DATA_FROM_ISERDES(007),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(07),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_08 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")
      PORT MAP (
         Q1 => DATA_FROM_ISERDES(056),
         Q2 => DATA_FROM_ISERDES(040),
         Q3 => DATA_FROM_ISERDES(024),
         Q4 => DATA_FROM_ISERDES(008),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(08),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_09 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(057),
         Q2 => DATA_FROM_ISERDES(041),
         Q3 => DATA_FROM_ISERDES(025),
         Q4 => DATA_FROM_ISERDES(009),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(09),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_10 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")
      PORT MAP (
         Q1 => DATA_FROM_ISERDES(058),
         Q2 => DATA_FROM_ISERDES(042),
         Q3 => DATA_FROM_ISERDES(026),
         Q4 => DATA_FROM_ISERDES(010),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(10),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_11 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")
      PORT MAP (
         Q1 => DATA_FROM_ISERDES(059),
         Q2 => DATA_FROM_ISERDES(043),
         Q3 => DATA_FROM_ISERDES(027),
         Q4 => DATA_FROM_ISERDES(011),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(11),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_12 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")
      PORT MAP (
         Q1 => DATA_FROM_ISERDES(060),
         Q2 => DATA_FROM_ISERDES(044),
         Q3 => DATA_FROM_ISERDES(028),
         Q4 => DATA_FROM_ISERDES(012),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(12),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_13 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(061),
         Q2 => DATA_FROM_ISERDES(045),
         Q3 => DATA_FROM_ISERDES(029),
         Q4 => DATA_FROM_ISERDES(013),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(13),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_14 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")

      PORT MAP (
         Q1 => DATA_FROM_ISERDES(062),
         Q2 => DATA_FROM_ISERDES(046),
         Q3 => DATA_FROM_ISERDES(030),
         Q4 => DATA_FROM_ISERDES(014),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(14),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

   ISERDES_RX_DATA_15 : ISERDES_NODELAY 
   GENERIC MAP(
         BITSLIP_ENABLE => TRUE, DATA_RATE => "SDR", DATA_WIDTH => 4, 
         INTERFACE_TYPE => "NETWORKING", NUM_CE => 1, SERDES_MODE =>"MASTER")
      PORT MAP (
         Q1 => DATA_FROM_ISERDES(063),
         Q2 => DATA_FROM_ISERDES(047),
         Q3 => DATA_FROM_ISERDES(031),
         Q4 => DATA_FROM_ISERDES(015),
         Q5 => open,
         Q6 => open,
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         BITSLIP => '0',
         CE1 => '1',
         CE2 => '0',
         CLK => RXCLK_TEMP,
         CLKB => RXCLK_TMP_INV,
         CLKDIV => RXCLKDIV_TEMP,
         D => DATA_RX_BUF(15),
         OCLK => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         RST => RESET);   
   

END SDR_4TO1_16CHAN_RX;
