--------------------------------------------------------------------------------
-- -- Copyright (c) 2009 Xilinx, Inc.
-- -- This design is confidential and proprietary of Xilinx, All Rights
-- Reserved.
--------------------------------------------------------------------------------
-- -   ____  ____
-- -  /   /\/   /
-- - /___/  \  /   Vendor: Xilinx
-- - \   \   \/    Version: 1.0
-- -  \   \        Filename: SDR_4TO1_16CHAN_TX.vhd
-- -  /   /        
-- - /___/   /\    Date Created: 07/14/2009 
-- - \   \  /  \   
-- -  \___\/\___\
-- - 
-- -Revision History:
--------------------------------------------------------------------------------
-- 
-- Description of module:
--
-- The SDR_4TO1_16CHAN_TX module contains all components in the LVDS 
-- Transmitter,including 16 channels of LVDS data and one channel of LVDS clock.
-- 
--------------------------------------------------------------------------------

----
--
-- Library declarations
--
-- Standard IEEE libraries
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library unisim;
use unisim.vcomponents.all;
--


ENTITY SDR_4TO1_16CHAN_TX IS
   PORT (
      DATA_TX_P               : OUT std_logic_vector(15 DOWNTO 0);   -- SERIAL SIDE TX DATA (P)
      DATA_TX_N               : OUT std_logic_vector(15 DOWNTO 0);   -- SERIAL SIDE TX DATA (N)
      CLOCK_TX_P              : OUT std_logic;   -- FORWARDED CLOCK TO RX (P)
      CLOCK_TX_N              : OUT std_logic;   -- FORWARDED CLOCK TO RX (N)
      TXCLK                   : IN std_logic;   -- SERIAL SIDE TX CLOCK
      TXCLKDIV                : IN std_logic;   -- PARALLEL SIDE TX CLOCK (DIVIDED FROM TXCLK)
      DATA_TO_OSERDES         : IN std_logic_vector(63 DOWNTO 0);   -- PARALLEL SIDE TX DATA
      RESET                   : IN std_logic);   -- TX DOMAIN RESET
END SDR_4TO1_16CHAN_TX;

ARCHITECTURE SDR_4TO1_16CHAN_TX OF SDR_4TO1_16CHAN_TX IS


   SIGNAL TX_CLOCK_PREBUF          :  std_logic;   
   SIGNAL TX_DATA_PREBUF           :  std_logic_vector(15 DOWNTO 0);   
  -- SIGNAL i                        :  integer;   

BEGIN
   
   --FORWARDED CLOCK	

   ODDR_TX_CLOCK : ODDR 
      GENERIC MAP(
         DDR_CLK_EDGE => "OPPOSITE_EDGE", INIT => '0', SRTYPE => "ASYNC")
      PORT MAP (
         Q => TX_CLOCK_PREBUF,
         C => TXCLK,
         CE => '1',
         D1 => '0',
         D2 => '1',
         R => '0',
         S => '0');   
   
   
   --FORWARDED CLOCK OUTPUT BUFFER

   OBUFDS_TX_CLOCK : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => CLOCK_TX_P,
         OB => CLOCK_TX_N,
         I => TX_CLOCK_PREBUF);   
   
   
   --MASTER OSERDES IN DATA PATH

   OSERDES_TX_DATA_00 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(00),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(000),
         D2 => DATA_TO_OSERDES(016),
         D3 => DATA_TO_OSERDES(032),
         D4 => DATA_TO_OSERDES(048),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_01 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(01),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(001),
         D2 => DATA_TO_OSERDES(017),
         D3 => DATA_TO_OSERDES(033),
         D4 => DATA_TO_OSERDES(049),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_02 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1)

      PORT MAP (
         OQ => TX_DATA_PREBUF(02),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(002),
         D2 => DATA_TO_OSERDES(018),
         D3 => DATA_TO_OSERDES(034),
         D4 => DATA_TO_OSERDES(050),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_03 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(03),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(003),
         D2 => DATA_TO_OSERDES(019),
         D3 => DATA_TO_OSERDES(035),
         D4 => DATA_TO_OSERDES(051),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_04 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(04),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(004),
         D2 => DATA_TO_OSERDES(020),
         D3 => DATA_TO_OSERDES(036),
         D4 => DATA_TO_OSERDES(052),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_05 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(05),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(005),
         D2 => DATA_TO_OSERDES(021),
         D3 => DATA_TO_OSERDES(037),
         D4 => DATA_TO_OSERDES(053),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_06 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(06),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(006),
         D2 => DATA_TO_OSERDES(022),
         D3 => DATA_TO_OSERDES(038),
         D4 => DATA_TO_OSERDES(054),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_07 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(07),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(007),
         D2 => DATA_TO_OSERDES(023),
         D3 => DATA_TO_OSERDES(039),
         D4 => DATA_TO_OSERDES(055),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_08 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(08),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(008),
         D2 => DATA_TO_OSERDES(024),
         D3 => DATA_TO_OSERDES(040),
         D4 => DATA_TO_OSERDES(056),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_09 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(09),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(009),
         D2 => DATA_TO_OSERDES(025),
         D3 => DATA_TO_OSERDES(041),
         D4 => DATA_TO_OSERDES(057),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_10 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(10),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(010),
         D2 => DATA_TO_OSERDES(026),
         D3 => DATA_TO_OSERDES(042),
         D4 => DATA_TO_OSERDES(058),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_11 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1)

      PORT MAP (
         OQ => TX_DATA_PREBUF(11),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(011),
         D2 => DATA_TO_OSERDES(027),
         D3 => DATA_TO_OSERDES(043),
         D4 => DATA_TO_OSERDES(059),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_12 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(12),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(012),
         D2 => DATA_TO_OSERDES(028),
         D3 => DATA_TO_OSERDES(044),
         D4 => DATA_TO_OSERDES(060),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_13 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1) 
      PORT MAP (
         OQ => TX_DATA_PREBUF(13),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(013),
         D2 => DATA_TO_OSERDES(029),
         D3 => DATA_TO_OSERDES(045),
         D4 => DATA_TO_OSERDES(061),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_14 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1)

      PORT MAP (
         OQ => TX_DATA_PREBUF(14),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(014),
         D2 => DATA_TO_OSERDES(030),
         D3 => DATA_TO_OSERDES(046),
         D4 => DATA_TO_OSERDES(062),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   

   OSERDES_TX_DATA_15 : OSERDES 
     GENERIC MAP(
        DATA_RATE_OQ => "SDR", DATA_RATE_TQ => "SDR",  DATA_WIDTH => 4, 
        INIT_OQ => '0', INIT_TQ => '0', SERDES_MODE => "MASTER", 
        SRVAL_OQ => '0', SRVAL_TQ => '0', TRISTATE_WIDTH => 1)
      PORT MAP (
         OQ => TX_DATA_PREBUF(15),
         SHIFTOUT1 => open,
         SHIFTOUT2 => open,
         TQ => open,
         CLK => TXCLK,
         CLKDIV => TXCLKDIV,
         D1 => DATA_TO_OSERDES(015),
         D2 => DATA_TO_OSERDES(031),
         D3 => DATA_TO_OSERDES(047),
         D4 => DATA_TO_OSERDES(063),
         D5 => '0',
         D6 => '0',
         OCE => '1',
         REV => '0',
         SHIFTIN1 => '0',
         SHIFTIN2 => '0',
         SR => RESET,
         T1 => '0',
         T2 => '0',
         T3 => '0',
         T4 => '0',
         TCE => '0');   
   
   
   --OUTPUT BUFFERS	

   OBUFDS_TX_DATA_00 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(00),
         OB => DATA_TX_N(00),
         I => TX_DATA_PREBUF(00));   
   

   OBUFDS_TX_DATA_01 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(01),
         OB => DATA_TX_N(01),
         I => TX_DATA_PREBUF(01));   
   

   OBUFDS_TX_DATA_02 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(02),
         OB => DATA_TX_N(02),
         I => TX_DATA_PREBUF(02));   
   

   OBUFDS_TX_DATA_03 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(03),
         OB => DATA_TX_N(03),
         I => TX_DATA_PREBUF(03));   
   

   OBUFDS_TX_DATA_04 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(04),
         OB => DATA_TX_N(04),
         I => TX_DATA_PREBUF(04));   
   

   OBUFDS_TX_DATA_05 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(05),
         OB => DATA_TX_N(05),
         I => TX_DATA_PREBUF(05));   
   

   OBUFDS_TX_DATA_06 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(06),
         OB => DATA_TX_N(06),
         I => TX_DATA_PREBUF(06));   
   

   OBUFDS_TX_DATA_07 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(07),
         OB => DATA_TX_N(07),
         I => TX_DATA_PREBUF(07));   
   

   OBUFDS_TX_DATA_08 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(08),
         OB => DATA_TX_N(08),
         I => TX_DATA_PREBUF(08));   
   

   OBUFDS_TX_DATA_09 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(09),
         OB => DATA_TX_N(09),
         I => TX_DATA_PREBUF(09));   
   

   OBUFDS_TX_DATA_10 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(10),
         OB => DATA_TX_N(10),
         I => TX_DATA_PREBUF(10));   
   

   OBUFDS_TX_DATA_11 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(11),
         OB => DATA_TX_N(11),
         I => TX_DATA_PREBUF(11));   
   

   OBUFDS_TX_DATA_12 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(12),
         OB => DATA_TX_N(12),
         I => TX_DATA_PREBUF(12));   
   

   OBUFDS_TX_DATA_13 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(13),
         OB => DATA_TX_N(13),
         I => TX_DATA_PREBUF(13));   
   

   OBUFDS_TX_DATA_14 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(14),
         OB => DATA_TX_N(14),
         I => TX_DATA_PREBUF(14));   
   

   OBUFDS_TX_DATA_15 : OBUFDS_LVDSEXT_25 
      PORT MAP (
         O => DATA_TX_P(15),
         OB => DATA_TX_N(15),
         I => TX_DATA_PREBUF(15));   
   

END SDR_4TO1_16CHAN_TX;
