// -----------------------------------------------------------------------------
// -- Copyright (c) 2009 Xilinx, Inc.
// -- This design is confidential and proprietary of Xilinx, All Rights
// Reserved.
// -----------------------------------------------------------------------------
// -   ____  ____
// -  /   /\/   /
// - /___/  \  /   Vendor: Xilinx
// - \   \   \/    Version: 1.0
// -  \   \        Filename: PRBSGen.v
// -  /   /        
// - /___/   /\    Date Created: 07/14/2009 
// - \   \  /  \   
// -  \___\/\___\
// - 
// -Revision History:
// -----------------------------------------------------------------------------
/*
--------------------------------------------------------------------------------
Description of module:

The PRBSGen module contains the four individual types of PRBS Generator,
which are PRBS7, PRBS15, PRBS23, and PRBS29.  Each of the modules below
call on LFSR's for the specific types of PRBS, advancing the patterns
8 bits on every clock cycle.
--------------------------------------------------------------------------------
*/


//-------------------------------------------------------------------------
// 7-bit Pseudo Random Pattern
//   :
//   : Shifter Stages  :  7
//   : Length          :  2^7 - 1 = 127 bits
//   : Longest Sequence:  5 (non-inverted)
//   : Poly            :  x^7 + x^6
//   :
//   : 000 0011
//   :   0    3
//-------------------------------------------------------------------------
module PRBSGen07
	(
	data_out, 
	advance_in, 
	reset_in,
	poly_in, 
	length_in, 
	clock_in
	);

output	[03:00]	data_out;
input	[06:00]	poly_in;
input		length_in, advance_in, reset_in, clock_in;

reg	[06:00]	PRBS;
reg 	[03:00]	data_out;

wire	[06:00]	ix00, ix01, ix02, ix03;//, ix04, ix05;// ix06, ix07;

IX07 intermediate_expression00(ix00, PRBS, poly_in, length_in);
IX07 intermediate_expression01(ix01, ix00, poly_in, length_in);
IX07 intermediate_expression02(ix02, ix01, poly_in, length_in);
IX07 intermediate_expression03(ix03, ix02, poly_in, length_in);
//IX07 intermediate_expression04(ix04, ix03, poly_in, length_in);
//IX07 intermediate_expression05(ix05, ix04, poly_in, length_in);
//IX07 intermediate_expression06(ix06, ix05, poly_in, length_in);
//IX07 intermediate_expression07(ix07, ix06, poly_in, length_in);

always @ (posedge clock_in) begin
   if (reset_in) begin 
      PRBS <= 7'b1111111;
      data_out <= 1;
   end else if (advance_in) begin
      PRBS <= ix03;
      data_out <= {ix00[06], ix01[06], ix02[06], ix03[06]};//, ix04[06], ix05[06]};//, ix06[06], ix07[06]};
   end
end
endmodule


//-------------------------------------------------------------------------
// ITU-T Recommendation O.150 Section 5.3
//   :
//   : Shifter Stages  :  15
//   : Length          :  2^15 - 1 = 32,767 bits
//   : Longest Sequence:  15 (inverted)
//   : Poly            :  x^15 + x^14
//   :
//   : 000 0000 0000 0011
//   :   0    0    0    3
//-------------------------------------------------------------------------
module PRBSGen15
	(
	data_out, 
	advance_in, 
	reset_in,
	poly_in, 
	length_in, 
	clock_in
	);

   output [03:00] data_out;
   input [14:00]  poly_in;
   input          length_in, advance_in, reset_in, clock_in;
   
   reg [14:00]    PRBS;
   reg [03:00]    data_out;
   
   wire [14:00]   ix00, ix01, ix02, ix03;//, ix04, ix05;// ix06, ix07;

   IX15 intermediate_expression00(ix00, PRBS, poly_in, length_in);
   IX15 intermediate_expression01(ix01, ix00, poly_in, length_in);
   IX15 intermediate_expression02(ix02, ix01, poly_in, length_in);
   IX15 intermediate_expression03(ix03, ix02, poly_in, length_in);
   //IX15 intermediate_expression04(ix04, ix03, poly_in, length_in);
   //IX15 intermediate_expression05(ix05, ix04, poly_in, length_in);
   //IX15 intermediate_expression06(ix06, ix05, poly_in, length_in);
   //IX15 intermediate_expression07(ix07, ix06, poly_in, length_in);

   always @ (posedge clock_in) begin
      if (reset_in) begin 
         PRBS <= 15'b111111111111111;
         data_out <= 1;
      end else if (advance_in) begin
         PRBS <= ix03;
         data_out <= {ix00[14], ix01[14], ix02[14], ix03[14]};//, ix04[14], ix05[14]};//, ix06[14], ix07[14]};
      end
   end
endmodule


//-------------------------------------------------------------------------
// ITU-T Recommendation O.150 Section 5.6
//   :
//   : Shifter Stages  :  23
//   : Length          :  2^23 - 1 = 8,388,607 bits
//   : Longest Sequence:  23 (inverted)
//   : Poly            :  x^23 + x^18
//   :
//   : 000 0000 0000 0000 0010 0001
//   :   0    0    0    0    2    1
//-------------------------------------------------------------------------
module PRBSGen23
	(
	data_out, 
	advance_in, 
	reset_in,
	poly_in, 
	length_in, 
	clock_in
        );

output	[03:00]	data_out;
input 	[22:00]	poly_in;
input		length_in, advance_in, reset_in, clock_in;

reg 	[22:00]	PRBS;
reg 	[03:00]	data_out;

wire 	[22:00]	ix00, ix01, ix02, ix03;//, ix04, ix05;// ix06, ix07;

IX23 intermediate_expression00(ix00, PRBS, poly_in, length_in);
IX23 intermediate_expression01(ix01, ix00, poly_in, length_in);
IX23 intermediate_expression02(ix02, ix01, poly_in, length_in);
IX23 intermediate_expression03(ix03, ix02, poly_in, length_in);
//IX23 intermediate_expression04(ix04, ix03, poly_in, length_in);
//IX23 intermediate_expression05(ix05, ix04, poly_in, length_in);
//IX23 intermediate_expression06(ix06, ix05, poly_in, length_in);
//IX23 intermediate_expression07(ix07, ix06, poly_in, length_in);

always @ (posedge clock_in) begin
   if (reset_in) begin 
      PRBS <= 23'b11111111111111111111111;
      data_out <= 1;
   end else if (advance_in) begin
      PRBS <= ix03;
      data_out <= {ix00[22], ix01[22], ix02[22], ix03[22]};//, ix06[22], ix07[22]};
   end
end
endmodule


//-------------------------------------------------------------------------
//	ITU-T Recommendation O.150 Section 5.7
//	:
//	: Shifter Stages  :  29
//	: Length          :  2^29 - 1 = 536,870,911 bits
//	: Longest Sequence:  29 (inverted)
//	: Poly            :  x^29 + x^27
//	:
//	: 0 0000 0000 0000 0000 0000 0000 0101
//	: 0    0    0    0    0    0    0    5
//-------------------------------------------------------------------------
module PRBSGen29
	(
	data_out, 
	advance_in, 
	reset_in,
	poly_in, 
	length_in, 
	clock_in
	);

   output [03:00] data_out;
   input [28:00]  poly_in;
   input          length_in, advance_in, reset_in, clock_in;
   
   reg [28:00]    PRBS;
   reg [03:00]    data_out;
   
   wire [28:00]   ix00, ix01, ix02, ix03;//, ix04, ix05;// ix06, ix07;
   
   IX29 intermediate_expression00(ix00, PRBS, poly_in, length_in);
   IX29 intermediate_expression01(ix01, ix00, poly_in, length_in);
   IX29 intermediate_expression02(ix02, ix01, poly_in, length_in);
   IX29 intermediate_expression03(ix03, ix02, poly_in, length_in);
   //IX29 intermediate_expression04(ix04, ix03, poly_in, length_in);
   //IX29 intermediate_expression05(ix05, ix04, poly_in, length_in);
   //IX29 intermediate_expression06(ix06, ix05, poly_in, length_in);
   //IX29 intermediate_expression07(ix07, ix06, poly_in, length_in);

   always @ (posedge clock_in) begin
      if (reset_in) begin 
         PRBS <= 29'b11111111111111111111111111111;
         data_out <= 1;
      end else if (advance_in) begin
         PRBS <= ix03;
         data_out <= {ix00[28], ix01[28], ix02[28], ix03[28]};//, ix04[28], ix05[28]};//, ix06[28], ix07[28]};
      end
   end
endmodule
