// -----------------------------------------------------------------------------
// -- Copyright (c) 2009 Xilinx, Inc.
// -- This design is confidential and proprietary of Xilinx, All Rights
// Reserved.
// -----------------------------------------------------------------------------
// -   ____  ____
// -  /   /\/   /
// - /___/  \  /   Vendor: Xilinx
// - \   \   \/    Version: 1.1
// -  \   \        Filename: SDR_4TO1_16CHAN_TOP.v
// -  /   /        
// - /___/   /\    Date Created: 07/14/2009 
// - \   \  /  \   
// -  \___\/\___\
// - 
// - Revision History:
// - Revision 1.1 06/28/2010 vasud
// - Updated MMCM Multiplier/Divider settings to match VCO frequency range
// -----------------------------------------------------------------------------
/*
--------------------------------------------------------------------------------
Description of module:

The SDR_4TO1_16CHAN_TOP module is the top level of the Hardware Test bench,
which serves as a test bench for the "interface under test", which is a 16
channel SFI LVDS interface.  This test bench includes pattern generators
and error detectors to test the integrity of the link and FIFO's to interface
to the receiver and transmitter.  Resets for the whole system are created in
this module.
--------------------------------------------------------------------------------
*/

`timescale 1ps/1ps
module SDR_4TO1_16CHAN_TOP
	(
	DATA_TX_P,
	DATA_TX_N,
	CLOCK_TX_P,
	CLOCK_TX_N,
	
	DATA_RX_P,
	DATA_RX_N,
	CLOCK_RX_P,
	CLOCK_RX_N,
	
	INTERFACE_CLK_P,
	INTERFACE_CLK_N,
	OSC1_200M_P,
	OSC1_200M_N,

	RESET,
	RESET_DETECTOR,
	USER_LED,
	ARMED_FLAG,
	LOCKED_FLAG,
	DISTRESS_FLAG,
	ABORT_FLAG,
	IDELAY_READY,
	ERRORED_BITS,
	FRAME_COUNT,
	TAP_COUNT,
	TRAINING_DONE,
	LOCKED_DCM,
	
	BERT_CAPTURE_FC,
	CAPTURE_TAP_COUNT,
	PAT_SEL,
	ERROR_INSERT,
	ERROR_COUNT_CHAN_SEL,
	TAP_COUNT_CHAN_SEL,

	CLK200_O,
	DEBUG_BUS,
	MANUAL_DELAY_INC,
	MANUAL_DELAY_DEC
	
	);
	
input	[15:0]	DATA_RX_P;		//SOURCE SYNC DATA INPUT (P)
input	[15:0]	DATA_RX_N;		//SOURCE SYNC DATA INPUT (N)
input		CLOCK_RX_P;		//SOURCE SYNC CLOCK INPUT (P)
input		CLOCK_RX_N;		//SOURCE SYNC CLOCK INPUT (N)

input           OSC1_200M_P;            //IDLYCTRL CLOCK
input           OSC1_200M_N;            //IDLYCTRL CLOCK
input		INTERFACE_CLK_P;	//TX CLOCK SOURCE FROM CLKMOD2 (P)
input		INTERFACE_CLK_N;	//TX CLOCK SOURCE FROM CLKMOD2 (N)
input		RESET;			//ALL RESETS CASCADE FROM THIS RESET
input		RESET_DETECTOR;		//RESETS ONLY THE ERROR DETECTORS
input		ERROR_INSERT;           //FORCES ERRORS ACROSS 16 CHANNELS
input		BERT_CAPTURE_FC;	//STROBE TO CAPTURE FRAME_COUNT INTO SLOWER DOMAIN
input		CAPTURE_TAP_COUNT;	//STROBE TO CAPTURE TAP_COUNT INTO SLOWER DOMAIN		
input	[3:0]	PAT_SEL;		//SELECTION OF TEST DATA PATTERN (e.g. PRBS23)
input	[3:0]	ERROR_COUNT_CHAN_SEL;	//SELECTS CHANNEL TO READ ERROR COUNT FROM
input	[4:0]	TAP_COUNT_CHAN_SEL;	//SELECTS CHANNEL TO READ TAP COUNT FROM
input		MANUAL_DELAY_INC;	//MANUALLY INCREMENTS THE DATA DELAY
input		MANUAL_DELAY_DEC;       //MANUALLY DECREMENTS THE DATA DELAY                                

output		CLK200_O;
output	[15:0]	DATA_TX_P;		//SOURCE SYNC DATA OUTPUT (P) 
output	[15:0]	DATA_TX_N;              //SOURCE SYNC DATA OUTPUT (N) 
output		CLOCK_TX_P;             //SOURCE SYNC CLOCK OUTPUT (P)
output		CLOCK_TX_N;             //SOURCE SYNC CLOCK OUTPUT (N)
output	[6:1]	USER_LED;		//6 SYSTEM LED'S ON BOARD
output		ARMED_FLAG;		//ERROR DETECTOR STATE: ARMED
output		LOCKED_FLAG;            //ERROR DETECTOR STATE: LOCKED
output		DISTRESS_FLAG;		//ERROR DETECTOR STATE: DISTRESS
output		ABORT_FLAG;		//ERROR DETECTOR STATE: ABORT
output		IDELAY_READY;		//IDELAYCTRL CIRCUIT IS LOCKED
output	[31:0]	ERRORED_BITS;		//NUMBER OF BIT ERRORS ON CURRENT CHANNEL
output	[31:0]	FRAME_COUNT;		//FRAMES RECEIVED
output	[7:0]	TAP_COUNT;		//CURRENT IDELAY TAP SETTING (0-63)
output		TRAINING_DONE;		//TRAINING PROCESS IS COMPLETE
output		LOCKED_DCM;		//DCM DRIVING RESET LOGIC IS LOCKED
output  [95:0]  DEBUG_BUS;              //DEBUG SIGNALS TO CHIPSCOPE
wire		CLK200;
wire		RXCLK;
wire		RXCLKDIV;
wire	[63:0]	DATA_FROM_ISERDES;
wire	[63:0]	DATA_TO_OSERDES;
wire		TXCLK;

wire		TXCLKDIV;
(*KEEP = "TRUE"*)
wire		CLOCK_TX_BUF;
wire		ORST_PREREG;
wire		SYSRST_PREREG;
wire		IRST_PREREG;
wire            OSC_200M;
wire	[4:0]	TAP_00;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_01;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_02;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_03;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_04;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_05;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_06;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_07;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_08;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_09;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_10;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_11;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_12;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_13;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_14;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_15;		//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR EACH CHAN
wire	[4:0]	TAP_CLK;	//TAP COUNTER VALUE TO STORE IDELAY SETTING FOR CLOCK CHAN
wire	[15:0]	ARMED;
wire	[15:0]	LOCKED;
wire	[15:0]	DISTRESS;
wire	[15:0]	ABORT;

reg	[3:0]	SAMPLED_ERROR_INSERT = 0;
reg	[3:0]	SAMPLED_BERT_CAPTURE_FC = 0;
reg	[3:0]	RST_DETECTOR_CAPTURE = 0;
reg		BERT_ERROR_INSERT_PULSE = 0;
reg		BERT_CAPTURE_FC_PULSE = 0;
reg		RST_DETECTOR_PULSE = 0;
reg		ORST = 0;
reg		SYSRST = 0;
reg		IRST = 0;
reg	[4:0]	TAP_00_REG = 0;
reg	[4:0]	TAP_01_REG = 0;
reg	[4:0]	TAP_02_REG = 0;
reg	[4:0]	TAP_03_REG = 0;
reg	[4:0]	TAP_04_REG = 0;
reg	[4:0]	TAP_05_REG = 0;
reg	[4:0]	TAP_06_REG = 0;
reg	[4:0]	TAP_07_REG = 0;
reg	[4:0]	TAP_08_REG = 0;
reg	[4:0]	TAP_09_REG = 0;
reg	[4:0]	TAP_10_REG = 0;
reg	[4:0]	TAP_11_REG = 0;
reg	[4:0]	TAP_12_REG = 0;
reg	[4:0]	TAP_13_REG = 0;
reg	[4:0]	TAP_14_REG = 0;
reg	[4:0]	TAP_15_REG = 0;
reg	[4:0]	TAP_CLK_REG = 0;
reg     [7:0]   TAP_COUNT = 0;

assign		USER_LED[1] =	ABORT_FLAG;
assign		USER_LED[2] =   LOCKED_FLAG;
assign		USER_LED[3] =	TRAINING_DONE;
assign		USER_LED[4] =	| ERRORED_BITS;
assign		USER_LED[5] =	LOCKED_DCM;
assign		USER_LED[6] =	IDELAY_READY;

assign		ARMED_FLAG =	|ARMED;
assign		LOCKED_FLAG =	&LOCKED;
assign		DISTRESS_FLAG =	|DISTRESS;
assign		ABORT_FLAG =	|ABORT;

//200 MHZ REFCLK TO IDELAYCTRL

IBUFGDS #(
	.DIFF_TERM("TRUE"),
	.IOSTANDARD("LVDS_25")
) OSC_200_IN
	(
	.O(OSC_200M), 
	.I(OSC1_200M_P), 
	.IB(OSC1_200M_N)
	);

BUFG GLOBAL_CLK 
	(
	.O(CLK200), 
	.I(OSC_200M)
	);

//TRANSMITTER CLOCK INPUT
IBUFDS #(
	.DIFF_TERM("TRUE"),
	.IOSTANDARD("LVDS_25")
) TX_CLOCK_IN
	(
	.O(CLOCK_TX_BUF), 
	.I(INTERFACE_CLK_P), 
	.IB(INTERFACE_CLK_N)
	);


//CLOCK BUFFER FOR SERIAL SIDE CLOCK
BUFIO TX_CLK_BUFIO
	(
	.O(TXCLK), 
	.I(CLOCK_TX_BUF)
	);


//CLOCK BUFFER/DIVIDER FOR PARALLEL SIDE CLOCK
BUFR #(
	.BUFR_DIVIDE("4")
) TX_CLK_BUFR
	(
	.O(TXCLKDIV), 
	.CE(1'b1), 
	.CLR(1'b0), 
	.I(CLOCK_TX_BUF)
	);


//LVDS RECEIVER INTERFACE
SDR_4TO1_16CHAN_RX RX_IF			
	(
	.DATA_RX_P(DATA_RX_P),
	.DATA_RX_N(DATA_RX_N),
	.CLOCK_RX_P(CLOCK_RX_P),
	.CLOCK_RX_N(CLOCK_RX_N),
	.INC_PAD(MANUAL_DELAY_INC),
	.DEC_PAD(MANUAL_DELAY_DEC),
	.DATA_FROM_ISERDES(DATA_FROM_ISERDES),
	.RESET(IRST),
	.IDLY_RESET(1'b0),
	.IDELAYCTRL_RESET(ORST),
	.CLK200(CLK200),
	.TAP_00(TAP_00),
	.TAP_01(TAP_01),
	.TAP_02(TAP_02),
	.TAP_03(TAP_03),
	.TAP_04(TAP_04),
	.TAP_05(TAP_05),
	.TAP_06(TAP_06),
	.TAP_07(TAP_07),
	.TAP_08(TAP_08),
	.TAP_09(TAP_09),
	.TAP_10(TAP_10),
	.TAP_11(TAP_11),
	.TAP_12(TAP_12),
	.TAP_13(TAP_13),
	.TAP_14(TAP_14),
	.TAP_15(TAP_15),
	.TAP_CLK(TAP_CLK),
	.TRAINING_DONE(TRAINING_DONE),
	.RXCLK(RXCLK),
	.RXCLKDIV(RXCLKDIV),
	.IDELAY_READY(IDELAY_READY)
	);

//LVDS TRANSMITTER INTERFACE
SDR_4TO1_16CHAN_TX TX_IF			
	(
	.DATA_TX_P(DATA_TX_P),
	.DATA_TX_N(DATA_TX_N),
	.CLOCK_TX_P(CLOCK_TX_P),
	.CLOCK_TX_N(CLOCK_TX_N),
	.TXCLK(TXCLK),
	.TXCLKDIV(TXCLKDIV),
	.DATA_TO_OSERDES(DATA_TO_OSERDES),
	.RESET(ORST)
	);


//RESET MANAGEMENT

//CREATE ONE CYCLE OF LATENCY ON ALL RESETS TO EASE TIMING CLOSURE
always @(posedge TXCLKDIV)	ORST <= ORST_PREREG;
always @(posedge RXCLKDIV)	IRST <= IRST_PREREG;
always @(posedge CLK200)	SYSRST <= SYSRST_PREREG;


//PRODUCES RST SYNCHRONOUS TO TX LOGIC
RST_MACHINE RST_CTRL_TX	
 (
 .CLK_generic(TXCLKDIV),
 .RST_stimulus(~LOCKED_DCM),
 .IRDY(1'b1),
 .DOMAIN_RST(ORST_PREREG)
 );

 
//PRODUCES RST SYNCHRONOUS TO SYSTEM LOGIC
RST_MACHINE RST_CTRL_SYS	
 (
 .CLK_generic(CLK200),
 .RST_stimulus(~LOCKED_DCM),
 .IRDY(1'b1),
 .DOMAIN_RST(SYSRST_PREREG)
 ); 

 
//PRODUCES RST FOR RX LOGIC (MUST BE SYNCHRONOUS TO TX CLOCK, THOUGH) 
RST_MACHINE RST_CTRL_RX	
 (
 .CLK_generic(TXCLKDIV),
 .RST_stimulus((LOCKED_DCM & ~IDELAY_READY)),
 .IRDY(IDELAY_READY),
 .DOMAIN_RST(IRST_PREREG)
 );

//MMCM ADDED FOR RST PURPOSES
MMCM_BASE # (
	.BANDWIDTH ("OPTIMIZED"),
 	.CLKFBOUT_MULT_F (5.0),
	.DIVCLK_DIVIDE(1),
 	.CLKIN1_PERIOD (5.0),
 	.CLKOUT0_DIVIDE_F (5.0),
 	.CLKOUT1_DIVIDE (8)
)

MMCM_FOR_RESET(
	.CLKOUT0(),
	.CLKOUT1(),
	.CLKOUT2(),
	.CLKOUT3(),
	.CLKOUT4(),
	.CLKOUT5(),
	.CLKOUT6(),
	.CLKOUT0B(),
	.CLKOUT1B(),
	.CLKOUT2B(),
	.CLKOUT3B(),
	.CLKFBOUT(CLK_FB),
	.CLKFBOUTB(),
	.LOCKED(LOCKED_DCM),
	.PWRDWN(1'b0),
	.CLKIN1(OSC_200M),
	.CLKFBIN(CLK_FB_IN),
	.RST(RESET)
	);

BUFG BUFG_FB (.O(CLK_FB_IN), .I(CLK_FB));

//PATTERN GENERATION / ERROR DETECTION
SYSTEM_LOGIC SYSTEM
	(
	//LVDS INTERFACE DATA AND CLOCKS
	.DATA_TO_OSERDES(DATA_TO_OSERDES),
	.DATA_FROM_ISERDES(DATA_FROM_ISERDES),
	.RX_WRCLK(RXCLKDIV),
	.TX_RDCLK(TXCLKDIV),
	.TX_RST(ORST),
	.RX_RST(IRST),
	
	//SYSTEM CLOCK and CONTROLS
	.SYSCLK(CLK200),
	.SYSRST(SYSRST),
	.DETECTOR_RST(RST_DETECTOR_PULSE),
	.ERROR_INSERT(BERT_ERROR_INSERT_PULSE),
	.PAT_SEL(PAT_SEL),
	.CAPTURE_FC(BERT_CAPTURE_FC_PULSE),
	.ERROR_COUNT_CHAN_SEL(ERROR_COUNT_CHAN_SEL),
	.TRAINING_DONE(TRAINING_DONE),
        .IDELAY_READY(IDELAY_READY),  // Debugging
	
	//LVDS INTERFACE FLAGS AND STATS
	.ARMED_FLAG(ARMED),
	.LOCKED_FLAG(LOCKED),
	.DISTRESS_FLAG(DISTRESS),
	.ABORT_FLAG(ABORT),
	.ERRORED_BITS(ERRORED_BITS),
	.DATA_SAMPLED(DEBUG_BUS),
	.FRAME_COUNT(FRAME_COUNT)
	);


//SHORTEN RAW CONTROL SIGNALS TO ONE PULSE ON CLK200 DOMAIN
always @(posedge CLK200)
   begin : Remove_Metastability_SysClock
      // ASYNCHRONOUS ENTRY POINT
      SAMPLED_BERT_CAPTURE_FC[0] <= BERT_CAPTURE_FC ;
      SAMPLED_ERROR_INSERT[0]    <= ERROR_INSERT;
      RST_DETECTOR_CAPTURE[0] <= RESET_DETECTOR;
      // METASTABLE FLIP-FLOPS
      begin : cascade_sys
         integer I;
         for(I = 0; I <= 3 - 1; I = I + 1)
         begin : Rx_meta_FF
            SAMPLED_BERT_CAPTURE_FC[I + 1] <= SAMPLED_BERT_CAPTURE_FC[I];
	    SAMPLED_ERROR_INSERT[I + 1]   <= SAMPLED_ERROR_INSERT[I];
            RST_DETECTOR_CAPTURE[I + 1] <= RST_DETECTOR_CAPTURE[I];
         end
      end
      // PRODUCE ONE PULSE ONLY
      BERT_CAPTURE_FC_PULSE <= SAMPLED_BERT_CAPTURE_FC[2] & ~SAMPLED_BERT_CAPTURE_FC[3] ;
      BERT_ERROR_INSERT_PULSE <= SAMPLED_ERROR_INSERT[2] & ~SAMPLED_ERROR_INSERT[3];
      RST_DETECTOR_PULSE <= RST_DETECTOR_CAPTURE[2] & ~RST_DETECTOR_CAPTURE[3] ;
   end

//TAP COUNTS ARE CAPTURED WHENEVER THE "CAPTURE_TAP_COUNT" PIN IS ASSERTED.  
//THIS ALLOWS SLOWER CLOCK DOMAINS TO READ THE DATA WITHOUT DANGER OF METASTABILITY 
always @ (posedge RXCLKDIV) begin
	if (CAPTURE_TAP_COUNT) begin
		TAP_00_REG <= TAP_00;
		TAP_01_REG <= TAP_01;
		TAP_02_REG <= TAP_02;
		TAP_03_REG <= TAP_03;
		TAP_04_REG <= TAP_04;
		TAP_05_REG <= TAP_05;
		TAP_06_REG <= TAP_06;
		TAP_07_REG <= TAP_07;
		TAP_08_REG <= TAP_08;
		TAP_09_REG <= TAP_09;
		TAP_10_REG <= TAP_10;
		TAP_11_REG <= TAP_11;
		TAP_12_REG <= TAP_12;
		TAP_13_REG <= TAP_13;
		TAP_14_REG <= TAP_14;
		TAP_15_REG <= TAP_15;
		TAP_CLK_REG <= TAP_CLK;
	end
	else begin
		TAP_00_REG <= TAP_00_REG;
		TAP_01_REG <= TAP_01_REG;
		TAP_02_REG <= TAP_02_REG;
		TAP_03_REG <= TAP_03_REG;
		TAP_04_REG <= TAP_04_REG;
		TAP_05_REG <= TAP_05_REG;
		TAP_06_REG <= TAP_06_REG;
		TAP_07_REG <= TAP_07_REG;
		TAP_08_REG <= TAP_08_REG;
		TAP_09_REG <= TAP_09_REG;
		TAP_10_REG <= TAP_10_REG;
		TAP_11_REG <= TAP_11_REG;
		TAP_12_REG <= TAP_12_REG;
		TAP_13_REG <= TAP_13_REG;
		TAP_14_REG <= TAP_14_REG;
		TAP_15_REG <= TAP_15_REG;
		TAP_CLK_REG <= TAP_CLK_REG;
	end
end

//READ IDELAY TAP COUNTS ON EACH CHANNEL IN ROUND ROBIN FASHION
always @ (posedge CLK200) begin
	case (TAP_COUNT_CHAN_SEL)
	5'b00000: TAP_COUNT <= {3'b000, TAP_00_REG};
	5'b00001: TAP_COUNT <= {3'b000, TAP_01_REG};
	5'b00010: TAP_COUNT <= {3'b000, TAP_02_REG};
	5'b00011: TAP_COUNT <= {3'b000, TAP_03_REG};
	5'b00100: TAP_COUNT <= {3'b000, TAP_04_REG};
	5'b00101: TAP_COUNT <= {3'b000, TAP_05_REG};
	5'b00110: TAP_COUNT <= {3'b000, TAP_06_REG};
	5'b00111: TAP_COUNT <= {3'b000, TAP_07_REG};
	5'b01000: TAP_COUNT <= {3'b000, TAP_08_REG};
	5'b01001: TAP_COUNT <= {3'b000, TAP_09_REG};
	5'b01010: TAP_COUNT <= {3'b000, TAP_10_REG};
	5'b01011: TAP_COUNT <= {3'b000, TAP_11_REG};
	5'b01100: TAP_COUNT <= {3'b000, TAP_12_REG};
	5'b01101: TAP_COUNT <= {3'b000, TAP_13_REG};
	5'b01110: TAP_COUNT <= {3'b000, TAP_14_REG};
	5'b01111: TAP_COUNT <= {3'b000, TAP_15_REG};
	default: TAP_COUNT <= {3'b000, TAP_CLK_REG};
	endcase
end

assign		CLK200_O = CLK200;

endmodule


