// -----------------------------------------------------------------------------
// -- Copyright (c) 2009 Xilinx, Inc.
// -- This design is confidential and proprietary of Xilinx, All Rights
// Reserved.
// -----------------------------------------------------------------------------
// -   ____  ____
// -  /   /\/   /
// - /___/  \  /   Vendor: Xilinx
// - \   \   \/    Version: 1.0
// -  \   \        Filename: SFI4_ML605_DEMO.v
// -  /   /        
// - /___/   /\    Date Created: 07/14/2009 
// - \   \  /  \   
// -  \___\/\___\
// - 
// -Revision History:
// -----------------------------------------------------------------------------
/*
--------------------------------------------------------------------------------
Description of module:

This module consists of the Chip scope cores to control and monitor the Hardware 
Testbench signals.

--------------------------------------------------------------------------------
*/

`timescale 1ps/1ps
module SFI4_ML605_DEMO
	(
	DATA_TX_P,
	DATA_TX_N,
	CLOCK_TX_P,
	CLOCK_TX_N,
	
	DATA_RX_P,
	DATA_RX_N,
	CLOCK_RX_P,
	CLOCK_RX_N,
	
	INTERFACE_CLK_P,
	INTERFACE_CLK_N,
	OSC1_200M_P,
	OSC1_200M_N,

	USER_LED
	
	);
	
input	[15:0]	DATA_RX_P;		// SOURCE SYNC DATA INPUT (P)
input	[15:0]	DATA_RX_N;		// SOURCE SYNC DATA INPUT (N)
input		CLOCK_RX_P;		// SOURCE SYNC CLOCK INPUT (P)
input		CLOCK_RX_N;		// SOURCE SYNC CLOCK INPUT (N)

input           OSC1_200M_P;            // IDLYCTRL CLOCK
input           OSC1_200M_N;            // IDLYCTRL CLOCK
input		INTERFACE_CLK_P;	// TX CLOCK SOURCE FROM CLKMOD2 (P)
input		INTERFACE_CLK_N;	// TX CLOCK SOURCE FROM CLKMOD2 (N)
output	[15:0]	DATA_TX_P;		// SOURCE SYNC DATA OUTPUT (P) 
output	[15:0]	DATA_TX_N;              // SOURCE SYNC DATA OUTPUT (N) 
output		CLOCK_TX_P;             // SOURCE SYNC CLOCK OUTPUT (P)
output		CLOCK_TX_N;             // SOURCE SYNC CLOCK OUTPUT (N)
output	[6:1]	USER_LED;		// 6 SYSTEM LED'S ON BOARD

wire		RESET;			// ALL RESETS CASCADE FROM THIS RESET
wire		RESET_DETECTOR;		// RESETS ONLY THE ERROR DETECTORS
wire		CAPTURE_FC;		// STROBE TO CAPTURE FRAME_COUNT INTO SLOWER DOMAIN
wire		CAPTURE_TAP_COUNT;	// STROBE TO CAPTURE TAP_COUNT INTO SLOWER DOMAIN		
wire	[3:0]	PAT_SEL;		// SELECTION OF TEST DATA PATTERN (e.g. PRBS23)
wire 		ERROR_INSERT;           // FORCES ERRORS ACROSS 16 CHANNELS
wire	[3:0]	ERROR_COUNT_CHAN_SEL;	// SELECTS CHANNEL TO READ ERROR COUNT FROM
wire	[4:0]	TAP_COUNT_CHAN_SEL;	// SELECTS CHANNEL TO READ TAP COUNT FROM
wire		MANUAL_DELAY_INC;	// MANUALLY INCREMENTS THE DATA DELAY
wire		MANUAL_DELAY_DEC;       // MANUALLY DECREMENTS THE DATA DELAY   
wire	[7:0]	TAP_COUNT;		// CURRENT IDELAY TAP SETTING (0-63)
wire		ARMED_FLAG;		// ERROR DETECTOR STATE: ARMED
wire		LOCKED_FLAG;            // ERROR DETECTOR STATE: LOCKED
wire		ABORT_FLAG;		// ERROR DETECTOR STATE: ABORT
wire		IDELAY_READY;		// IDELAYCTRL CIRCUIT IS LOCKED
wire    [31:0]  FRAME_COUNT;            // NUMBER OF FRAMES RECEIVED
wire	[31:0]	ERRORED_BITS;

wire    [35:0]  CONTROL0;               // ICON signal
wire    [35:0]  CONTROL1;               // ICON signal
wire    [95:0]  DEBUG_BUS;              // ILA signal
wire    [127:0] SYNC_IN;		// VIO signal
wire    [31:0]  SYNC_OUT;		// VIO signal
wire            ASYNC_OUT;		// VIO signal


SDR_4TO1_16CHAN_TOP TOP
	(
	.DATA_TX_P            (DATA_TX_P),
	.DATA_TX_N            (DATA_TX_N),
	.CLOCK_TX_P           (CLOCK_TX_P),
	.CLOCK_TX_N           (CLOCK_TX_N),
	
	.DATA_RX_P            (DATA_RX_P),
	.DATA_RX_N            (DATA_RX_N),
	.CLOCK_RX_P           (CLOCK_RX_P),
	.CLOCK_RX_N           (CLOCK_RX_N),
	
	.OSC1_200M_P          (OSC1_200M_P),
	.OSC1_200M_N          (OSC1_200M_N),
	.INTERFACE_CLK_P      (INTERFACE_CLK_P),
	.INTERFACE_CLK_N      (INTERFACE_CLK_N),
	
	.RESET                (RESET),
	.RESET_DETECTOR       (RESET_DETECTOR), 
	.USER_LED             (USER_LED),
	.ARMED_FLAG           (ARMED_FLAG),
	.LOCKED_FLAG          (LOCKED_FLAG),
	.DISTRESS_FLAG        (DISTRESS_FLAG),
	.ABORT_FLAG           (ABORT_FLAG),
	.IDELAY_READY         (IDELAY_READY),
	.ERRORED_BITS         (ERRORED_BITS),
	.FRAME_COUNT          (FRAME_COUNT),
	.TAP_COUNT            (TAP_COUNT),
	.TRAINING_DONE        (TRAINING_DONE),
	.LOCKED_DCM           (LOCKED_DCM),
	
	.BERT_CAPTURE_FC      (CAPTURE_FC),
	.CAPTURE_TAP_COUNT    (CAPTURE_TAP_COUNT),
	.PAT_SEL              (PAT_SEL),   
	.ERROR_INSERT         (ERROR_INSERT),
	.ERROR_COUNT_CHAN_SEL (ERROR_COUNT_CHAN_SEL),
	.TAP_COUNT_CHAN_SEL   (TAP_COUNT_CHAN_SEL),

	.CLK200_O             (CLK200),
	.DEBUG_BUS            (DEBUG_BUS),
	.MANUAL_DELAY_INC     (MANUAL_DELAY_INC),
	.MANUAL_DELAY_DEC     (MANUAL_DELAY_DEC)
	
	);

assign MANUAL_DELAY_INC      = SYNC_OUT[0];
assign MANUAL_DELAY_DEC      = SYNC_OUT[1];
assign RESET_DETECTOR        = SYNC_OUT[2];
assign CAPTURE_FC 	     = SYNC_OUT[3];
assign CAPTURE_TAP_COUNT     = SYNC_OUT[5];
assign PAT_SEL               = SYNC_OUT[9:6];
assign ERROR_COUNT_CHAN_SEL  = SYNC_OUT[13:10];
assign TAP_COUNT_CHAN_SEL    = SYNC_OUT[18:14];
assign ERROR_INSERT          = SYNC_OUT[19];

assign RESET                 = ASYNC_OUT;

assign SYNC_IN[0]            = LOCKED_FLAG;
assign SYNC_IN[1]            = ARMED_FLAG;
assign SYNC_IN[2]            = ABORT_FLAG;
assign SYNC_IN[10:3]         = TAP_COUNT;
assign SYNC_IN[11]           = IDELAY_READY;
assign SYNC_IN[12]           = TRAINING_DONE;
assign SYNC_IN[44:13]        = FRAME_COUNT;
assign SYNC_IN[76:45]        = ERRORED_BITS;
assign SYNC_IN[86:77]        = 10'b0;
assign SYNC_IN[87]           = LOCKED_DCM;
assign SYNC_IN[127:88]       = 40'b0;

icon i_icon
    (
      .CONTROL0(CONTROL0),
      .CONTROL1(CONTROL1)
   );

vio i_vio (
    .CONTROL(CONTROL0), 
    .CLK(CLK200), 
    .ASYNC_OUT(ASYNC_OUT), 
    .SYNC_IN(SYNC_IN),
    .SYNC_OUT(SYNC_OUT)
);

ila i_ila
    (
      .CLK    (CLK200),
      .DATA   (DEBUG_BUS),
      .TRIG0  (1'b1),
      .CONTROL(CONTROL1)
    );

endmodule


