// -----------------------------------------------------------------------------
// -- Copyright (c) 2009 Xilinx, Inc.
// -- This design is confidential and proprietary of Xilinx, All Rights
// Reserved.
// -----------------------------------------------------------------------------
// -   ____  ____
// -  /   /\/   /
// - /___/  \  /   Vendor: Xilinx
// - \   \   \/    Version: 1.0
// -  \   \        Filename: SDR_4TO1_16CHAN_TOP_TB.v
// -  /   /        
// - /___/   /\    Date Created: 07/14/2009 
// - \   \  /  \   
// -  \___\/\___\
// - 
// -Revision History:
// -----------------------------------------------------------------------------
/*
--------------------------------------------------------------------------------
Description of module:

Simple Test Bench for Simulation.
--------------------------------------------------------------------------------
*/

`timescale 1ps/1ps
module SDR_4TO1_16CHAN_TOP_TB ();

	

wire           	OSC1_200M_P;            // IDLYCTRL CLOCK
wire          	OSC1_200M_N;            // IDLYCTRL CLOCK
wire		INTERFACE_CLK_P;	// TX CLOCK SOURCE (P)
wire		INTERFACE_CLK_N;	// TX CLOCK SOURCE (N)
wire	[15:0]	DATA_TX_P;		// SOURCE SYNC DATA OUTPUT (P) 
wire	[15:0]	DATA_TX_N;              // SOURCE SYNC DATA OUTPUT (N) 
wire		CLOCK_TX_P;             // SOURCE SYNC CLOCK OUTPUT (P)
wire		CLOCK_TX_N;             // SOURCE SYNC CLOCK OUTPUT (N)
reg		CLK_200;                // IDLYCTRL CLOCK
reg		SERIAL_CLK;     	// TX CLOCK SOURCE
reg		IDELAY_READY_REG;
reg 		NEG_EDGE_FOUND;
wire		POS_IDELAY_READY;

reg		RESET;			// ALL RESETS CASCADE FROM THIS RESET
reg		BERT_CAPTURE_FC;	// STROBE TO CAPTURE FRAME_COUNT INTO SLOWER DOMAIN
reg		CAPTURE_TAP_COUNT;	// STROBE TO CAPTURE TAP_COUNT INTO SLOWER DOMAIN		
reg	[3:0]	PAT_SEL;		// SELECTION OF TEST DATA PATTERN (e.g. PRBS23)
reg 		ERROR_INSERT;           // FORCES ERRORS ACROSS 16 CHANNELS
reg	[3:0]	ERROR_COUNT_CHAN_SEL;	// SELECTS CHANNEL TO READ ERROR COUNT FROM
reg	[4:0]	TAP_COUNT_CHAN_SEL;	// SELECTS CHANNEL TO READ TAP COUNT FROM
reg		MANUAL_DELAY_INC;	// MANUALLY INCREMENTS THE DATA DELAY
reg		MANUAL_DELAY_DEC;       // MANUALLY DECREMENTS THE DATA DELAY   
wire	[7:0]	TAP_COUNT;		// CURRENT IDELAY TAP SETTING (0-63)
wire		ARMED_FLAG;		// ERROR DETECTOR STATE: ARMED
wire		LOCKED_FLAG;            // ERROR DETECTOR STATE: LOCKED
wire		ABORT_FLAG;		// ERROR DETECTOR STATE: ABORT
wire		IDELAY_READY;		// IDELAYCTRL CIRCUIT IS LOCKED
wire    [31:0]  FRAME_COUNT;            // NUMBER OF FRAMES RECEIVED
wire	[31:0]	ERRORED_BITS;
wire		COUNT_TC;
reg	[6:0]	COUNT = 7'b0;

localparam RST_SYNC_NUM = 3;

reg	[RST_SYNC_NUM-1:0] RST_DET_REG;
wire	RST_DETECTOR;


SDR_4TO1_16CHAN_TOP TOP
	(
	.DATA_TX_P            (DATA_TX_P),
	.DATA_TX_N            (DATA_TX_N),
	.CLOCK_TX_P           (CLOCK_TX_P),
	.CLOCK_TX_N           (CLOCK_TX_N),
	
	.DATA_RX_P            (DATA_TX_P),
	.DATA_RX_N            (DATA_TX_N),
	.CLOCK_RX_P           (CLOCK_TX_P),
	.CLOCK_RX_N           (CLOCK_TX_N),
	
	.OSC1_200M_P          (OSC1_200M_P),
	.OSC1_200M_N          (OSC1_200M_N),
	.INTERFACE_CLK_P      (INTERFACE_CLK_P),
	.INTERFACE_CLK_N      (INTERFACE_CLK_N),
	
	.RESET                (RESET),
	.RESET_DETECTOR       (RST_DETECTOR), 
	.USER_LED             (),
	.ARMED_FLAG           (ARMED_FLAG),
	.LOCKED_FLAG          (LOCKED_FLAG),
	.DISTRESS_FLAG        (DISTRESS_FLAG),
	.ABORT_FLAG           (ABORT_FLAG),
	.IDELAY_READY         (IDELAY_READY),
	.ERRORED_BITS         (ERRORED_BITS),
	.FRAME_COUNT          (FRAME_COUNT),
	.TAP_COUNT            (TAP_COUNT),
	.TRAINING_DONE        (TRAINING_DONE),
	.LOCKED_DCM           (LOCKED_DCM),
	
	.BERT_CAPTURE_FC      (BERT_CAPTURE_FC),
	.CAPTURE_TAP_COUNT    (CAPTURE_TAP_COUNT),
	.PAT_SEL              (PAT_SEL),   
	.ERROR_INSERT         (ERROR_INSERT),
	.ERROR_COUNT_CHAN_SEL (ERROR_COUNT_CHAN_SEL),
	.TAP_COUNT_CHAN_SEL   (TAP_COUNT_CHAN_SEL),

	.CLK200_O             (),
	.DEBUG_BUS	      (),
	.MANUAL_DELAY_INC     (MANUAL_DELAY_INC),
	.MANUAL_DELAY_DEC     (MANUAL_DELAY_DEC)
	
	);

// IDLYCTRL clock generation
  initial
     CLK_200 = 1'b0;
  always
     # 2500 CLK_200 = ~CLK_200;
  assign OSC1_200M_P = CLK_200;
  assign OSC1_200M_N = ~CLK_200;

// SERDES serial clock generation
  initial
     SERIAL_CLK = 1'b0;
  always
     # 700  SERIAL_CLK = ~ SERIAL_CLK;
  assign INTERFACE_CLK_P =  SERIAL_CLK;
  assign INTERFACE_CLK_N = ~ SERIAL_CLK;

// RESET generation
 initial 
 begin
 RESET = 1'b1;
 repeat (6) @ (posedge CLK_200);
 RESET = 1'b0;
 @ (posedge LOCKED_FLAG)
 repeat (200) @ (posedge CLK_200);
 RESET = 1'b1;
 repeat (6) @ (posedge CLK_200);
 RESET = 1'b0;
 end

 initial
 begin
 PAT_SEL = 4'b0101;  //PRBS23
 TAP_COUNT_CHAN_SEL = 5'b11111;
 ERROR_COUNT_CHAN_SEL = 4'b0100;
 MANUAL_DELAY_INC = 1'b0;
 MANUAL_DELAY_DEC = 1'b0;
 ERROR_INSERT = 1'b0;
 CAPTURE_TAP_COUNT = 1'b1;
 end

 always @ (posedge CLK_200)
 begin
   if (COUNT_TC)
     COUNT = 0;
   else if (LOCKED_FLAG)
     COUNT = COUNT + 1;
 end

assign COUNT_TC = (COUNT == 'd100);

 always @ (posedge CLK_200)
 begin
 if (COUNT_TC)
   begin
   BERT_CAPTURE_FC = 1'b1;
   end
 else
   begin
   BERT_CAPTURE_FC = 1'b0;
   end
 end
 
// ERROR DETECTOR RESET  
always @ (posedge CLK_200)
    IDELAY_READY_REG <= IDELAY_READY;

assign NEG_IDELAY_READY = (~IDELAY_READY) & IDELAY_READY_REG;

always @ (posedge CLK_200)
    if (NEG_IDELAY_READY)
      NEG_EDGE_FOUND <= 1'b1;
    else if (POS_IDELAY_READY) 
      NEG_EDGE_FOUND <= 1'b0;
    
assign POS_IDELAY_READY = IDELAY_READY & (~IDELAY_READY_REG) & NEG_EDGE_FOUND;

always @(posedge CLK_200 or posedge POS_IDELAY_READY)
    if (POS_IDELAY_READY)
      RST_DET_REG <= {RST_SYNC_NUM{1'b1}};
    else
      // logical left shift by one (pads with 0)
      RST_DET_REG <= RST_DET_REG << 1;

assign RST_DETECTOR    = RST_DET_REG[RST_SYNC_NUM-1];

endmodule


